/*
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package jchess;

import java.awt.*;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.Iterator;

/** Class to represent chessboard. Chessboard is made from squares.
 */
public class Chessboard  extends Canvas {
    public Square squares[][];//squares of chessboard
    Image image;//image of chessboard
    Image sel_square;//selected square
    Image able_square;
    public Square activeSquare;
    private int active_x_square;
    private int active_y_square;
    private float square_height;//height of square
    //public Graphics graph;
    public static final int img_x         = 5;//image x position (used in JChessView class!)
    public static final int img_y         = img_x;//image y position (used in JChessView class!)
    public static final int img_widht     = 480;//image width
    public static final int img_height    = img_widht;//image height
    private ArrayList moves;
    private Settings settings;
    public King kingWhite;
    public King kingBlack;
    
    //-------- for undo ----------
    private Square  undo1_sq_begin      = null;
    private Square  undo1_sq_end        = null;
    private Piece   undo1_piece_begin   = null;
    private Piece   undo1_piece_end     = null;
    private Piece   ifWasEnPassant      = null;
    private Piece   ifWasCastling       = null;
    private boolean breakCastling       = false; //if last move break castling
    //----------------------------
    
    //For En passant:
    //|-> Pawn whose in last turn moved two square
    public Pawn twoSquareMovedPawn=null;
    public Pawn twoSquareMovedPawn2=null;
    private Moves moves_history;
    
    public Chessboard(Settings settings, Moves moves_history) {
        this.image          = GUI.loadImage("chessboard.png");//image of chessboard
        this.sel_square     = GUI.loadImage("sel_square.png");//image of highlited square
        this.able_square    = GUI.loadImage("able_square.png");//image of square where piece can go
        //this.graph        = this.getGraphics();
        this.settings       = settings;
        this.activeSquare   = null;
        this.square_height  = img_height/8;//we need to devide to know height of field
        this.squares = new Square[8][8];//initalization of 8x8 chessboard
        this.active_x_square = 0;
        this.active_y_square = 0;
        for(int i=0; i<8; i++) {//create object for each square
            for(int y=0; y<8; y++) {
                this.squares[i][y] = new Square(i,y,null);
            }
        }//--endOf--create object for each square
        this.moves_history = moves_history;
    }/*--endOf-Chessboard--*/
    
          /* Method setPieces on begin of new game or loaded game
          * @places: string with pieces to set on chessboard
          */
    public void setPieces(String places, Player plWhite, Player plBlack) {
        
         if(places.equals("")) {//if newGame
             if(this.settings.upsideDown) {
                 this.setPieces4NewGame(true, plWhite, plBlack);
             } else {
                 this.setPieces4NewGame(false, plWhite, plBlack);
             }
             
         } else {//if loadedGame
             return;
         }
    }/*--endOf-setPieces--*/
    
        /*  method sets Pieces for newGame
         *  @upsideDown: if true white pieces will be on top of chessboard
         * */
    private void setPieces4NewGame(boolean upsideDown, Player plWhite, Player plBlack) {
        
        /* WHITE PIECES */
        Player player = plBlack;
        Player player1= plWhite;
        if(upsideDown) { //if white on Top
            player = plWhite;
            player1= plBlack;
        }
        this.setFigures4NewGame(0, player,  upsideDown);
        this.setPawns4NewGame  (1, player);
        this.setFigures4NewGame(7, player1, upsideDown);
        this.setPawns4NewGame  (6, player1);
    }/*--endOf-setPieces(boolean upsideDown)--*/
    
        /*  method set Figures in row (and set Queen and King to right position)
         *  @i: row where to set figures (Rook, Knight etc.)
         *  @player: player which is owner of pawns
         *  @upsideDown: if true white pieces will be on top of chessboard
         * */
    private void setFigures4NewGame(int i, Player player, boolean upsideDown ) {
       
        if(i != 0 && i != 7) {
            System.out.println("error setting figures like rook etc.");
            return;
        } else if(i==0) { 
            player.goDown = true;
        }
        
        this.squares[0][i].setPiece(new Rook  (this,player));
        this.squares[7][i].setPiece(new Rook  (this,player));
        this.squares[1][i].setPiece(new Knight(this,player));
        this.squares[6][i].setPiece(new Knight(this,player));
        this.squares[2][i].setPiece(new Bishop(this,player));
        this.squares[5][i].setPiece(new Bishop(this,player));
        if(upsideDown) {
            this.squares[4][i].setPiece(new Queen (this,player));
            if(player.color == Player.colors.white) {
                this.squares[3][i].setPiece(kingWhite = new King  (this,player));
            }
            else {
                this.squares[3][i].setPiece(kingBlack = new King  (this,player));
            }
        } else {
            this.squares[3][i].setPiece(new Queen (this,player));
            if(player.color == Player.colors.white) {
                this.squares[4][i].setPiece(kingWhite = new King  (this,player));
            }
            else {
                this.squares[4][i].setPiece(kingBlack = new King  (this,player));
            }
        }        
    }
    
        /*  method set Pawns in row
         *  @i: row where to set pawns
         *  @player: player which is owner of pawns
         * */   
    private void setPawns4NewGame(int i, Player player) {
        
        if(i!=1 && i!=6) {
            System.out.println("error setting pawns etc.");
            return;
        }
        for(int x=0;x<8;x++) {
            this.squares[x][i].setPiece(new Pawn(this,player));
        }        
    }

    public Square getSquare(int x, int y)
    {
        if((x > Chessboard.img_height) || (y > Chessboard.img_widht)) {//test if click is out of chessboard
            System.out.println("click out of chessboard.");
            return null;
        }
        double square_x = x/square_height;//count which field in X was clicked
        double square_y = y/square_height;//count which field in Y was clicked
       
        if(square_x  > (int)square_x) {//if X is more than X parsed to Integer
            square_x = (int)square_x + 1;//parse to integer and increment
        }
        if(square_y  > (int)square_y) {//if X is more than X parsed to Integer
            square_y = (int)square_y + 1;//parse to integer and increment
        }
        //Square newActiveSquare = this.squares[(int)square_x-1][(int)square_y-1];//4test
        System.out.println("square_x: "+square_x+" square_y: "+square_y+" \n"); //4tests
        Square result;
        try {
            result = this.squares[(int)square_x-1][(int)square_y-1];
        } catch(java.lang.ArrayIndexOutOfBoundsException exc) {
            System.out.println("!!Array out of bounds when getting Square with Chessboard.getSquare(int,int) : "+exc);
            return null;
        }
        return this.squares[(int)square_x-1][(int)square_y-1];
    }
        /* Method selecting piece in chessboard
         * @x : x position of mouse (when clicked)
         * @y : y position of mouse (when clicked)
         */
    public void select(Square sq) {
        

        this.activeSquare = sq;
        this.active_x_square = sq.pozX+1;
        this.active_y_square = sq.pozY+1;

        //this.draw();//redraw
        repaint();
        System.out.println("active_x: "+this.active_x_square+" active_y: "+this.active_y_square);//4tests
        
    }/*--endOf-select--*/
        /* Method set variables active_x_square & active_y_square
         * to 0 values.
         */
    void unselect() {
        
        this.active_x_square = 0;
        this.active_y_square = 0;
        this.activeSquare = null;
        //this.draw();//redraw
        repaint();
    }/*--endOf-unselect--*/

    int get_widht() {
        return img_widht;
    }/*--endOf-get_widht--*/

    int get_height() {
        return img_height;
    }/*--endOf-get_height--*/

    int get_square_height() {
        int result = (int)this.square_height;
        return result;
    }

    public void move(Square begin, Square end) { move(begin, end, true); }

    public void move(int xFrom, int yFrom, int xTo, int yTo) {
        Square fromSQ = null;
        Square toSQ   = null;
        try {
            fromSQ = this.squares[xFrom][yFrom];
            toSQ   = this.squares[xTo][yTo];
        } catch(java.lang.IndexOutOfBoundsException exc) {
            System.out.println("error moving piece: "+exc);
            return;
        }
        this.move(this.squares[xFrom][yFrom], this.squares[xTo][yTo], true);
    }
        /* Method move piece from square to square
         * @begin: begin square from which move piece
         * @end: end square where we want to move piece         *
         * @refresh: refresh chessboard, default: true
         * */
    public void move(Square begin, Square end, boolean refresh) 
    {
        
       //if(begin.piece.canMove(end, this.moves)) {
            if(end.piece != null) {
                end.piece.square = null;
            }
            Square tempBegin = new Square(begin);//4 moves history
            Square tempEnd   = new Square(end);  //4 moves history
            //for undo
            undo1_piece_begin   = begin.piece;
            undo1_sq_begin      = begin;
            undo1_piece_end     = end.piece;
            undo1_sq_end        = end;
            ifWasEnPassant      = null;
            ifWasCastling       = null;
            breakCastling       = false;
            // ---

            twoSquareMovedPawn2 = twoSquareMovedPawn;
            
            begin.piece.square  = end;//set square of piece to ending
            end.piece           = begin.piece;//for ending square set piece from beginin square
            begin.piece         = null;//make null piece for begining square
            
            if(end.piece.name.equals("King"))
            {
                if(!((King)end.piece).wasMotion)
                {
                    breakCastling = true;
                    ((King)end.piece).wasMotion = true;
                }

                //Castling
                if(begin.pozX+2 == end.pozX) 
                {
                    move(squares[7][begin.pozY], squares[end.pozX-1][begin.pozY], false);
                    ifWasCastling = end.piece;  //for undo
                    this.moves_history.addCastling("O-O");
                    return;
                }
                else if(begin.pozX-2 == end.pozX)
                {
                    move(squares[0][begin.pozY], squares[end.pozX+1][begin.pozY], false);
                    ifWasCastling = end.piece;  // for undo
                    this.moves_history.addCastling("O-O-O");
                    return;
                }
            } 
            else if(end.piece.name.equals("Rook"))
            {
                if(!((Rook)end.piece).wasMotion)
                {
                    breakCastling = true;
                    ((Rook)end.piece).wasMotion = true;
                }
            }
            else if(end.piece.name.equals("Pawn"))
            {
                if(twoSquareMovedPawn!=null && squares[end.pozX][begin.pozY]==twoSquareMovedPawn.square) //en passant
                {
                    ifWasEnPassant = squares[end.pozX][begin.pozY].piece; //for undo
                    squares[end.pozX][begin.pozY].piece=null;
                }

                if(begin.pozY-end.pozY == 2 || end.pozY-begin.pozY == 2) //moved two square
                {
                    breakCastling = true;
                    twoSquareMovedPawn = (Pawn)end.piece;
                }
                else
                {
                    twoSquareMovedPawn = null; //erase last saved move (for En passant)
                }
                
                if(end.piece.square.pozY == 0 ||end.piece.square.pozY==7 ) //promote Pawn
                {
                    String color;
                    if(end.piece.player.color == Player.colors.white) color = "W"; // promotionWindow was show with pieces in this color
                    else color = "B";

                    String newPiece = JChessApp.jcv.showPawnPromotionBox(color); //return name of new piece

                    if(newPiece.equals("Queen")) // transform pawn to queen
                    {
                        Queen queen         = new Queen (this,end.piece.player);
                        queen.chessboard    = end.piece.chessboard;
                        queen.player        = end.piece.player;
                        queen.square        = end.piece.square;
                        end.piece           = queen;
                    }
                    else if(newPiece.equals("Rook")) // transform pawn to rook
                    {
                        Rook rook       = new Rook (this,end.piece.player);
                        rook.chessboard = end.piece.chessboard;
                        rook.player     = end.piece.player;
                        rook.square     = end.piece.square;
                        end.piece       = rook;
                    }
                    else if(newPiece.equals("Bishop")) // transform pawn to bishop
                    {
                        Bishop bishop       = new Bishop (this,end.piece.player);
                        bishop.chessboard   = end.piece.chessboard;
                        bishop.player       = end.piece.player;
                        bishop.square       = end.piece.square;
                        end.piece           = bishop;
                    }
                    else // transform pawn to knight
                    {
                        Knight knight       = new Knight (this,end.piece.player);
                        knight.chessboard   = end.piece.chessboard;
                        knight.player       = end.piece.player;
                        knight.square       = end.piece.square;
                        end.piece           = knight;
                    }
                }
            }
            else if(!end.piece.name.equals("Pawn"))
            {
                twoSquareMovedPawn = null; //erase last saved move (for En passant)
            }
        //}
        
        if(refresh)
        {
            this.unselect();//unselect square
            //this.draw();//redraw
            repaint();
        }
        this.moves_history.addMove(tempBegin, tempEnd);
    }/*endOf-move()-*/
    
    public boolean undo(){ return undo(true); }
    
    public boolean undo(boolean refresh) //undo last move
    {
        return true;
        /*
        if(undo1_sq_begin != null)
        {
            Square begin = undo1_sq_begin;
            begin.piece = undo1_piece_begin;
            begin.piece.square = undo1_sq_begin;

            Square end = undo1_sq_end;
            end.piece = undo1_piece_end;
            if(end.piece!=null)
            {
                end.piece.square = undo1_sq_end;
            }

            if(begin.piece.name.equals("Rook")) 
            {
                if(ifWasCastling != null) //Castling
                {
                    ((Rook)begin.piece).wasMotion = false;
                    ((King)ifWasCastling).wasMotion = false;

                    this.squares[3][ifWasCastling.square.pozY].piece = ifWasCastling;
                    this.squares[ifWasCastling.square.pozX][ifWasCastling.square.pozY].piece = null;
                    ifWasCastling.square = squares[3][ifWasCastling.square.pozY];
                }
                else if(breakCastling == true)
                {
                    ((Rook)begin.piece).wasMotion = false;
                }
            }
            else if(begin.piece.name.equals("King") && breakCastling == true)
            {
                ((King)begin.piece).wasMotion = false;
            }
            else if(begin.piece.name.equals("Pawn") && ifWasEnPassant != null) //en passant
            {
                twoSquareMovedPawn = (Pawn)ifWasEnPassant;
                this.squares[ifWasEnPassant.square.pozX][ifWasEnPassant.square.pozY].piece = ifWasEnPassant;
            }

            if(twoSquareMovedPawn2 != null)
            {
                twoSquareMovedPawn = twoSquareMovedPawn2;
                twoSquareMovedPawn2 = null;
            }

            if(refresh)
            {
                this.unselect();//unselect square
                //this.draw();//redraw
                repaint();
            }

            undo1_sq_begin = null;
            undo1_sq_end = null;
            undo1_piece_begin = null;
            undo1_piece_end = null;
            ifWasEnPassant = null;
            ifWasCastling = null;
            breakCastling = false;

            return true;
        }
        else
        {
            return false;
        }*/
     
    }
        /* Method to draw Chessboard and their elements (pieces etc.)
         */
    void draw() {
        
        this.getGraphics().drawImage(image,0,0,null);//draw an Image of chessboard
        for(int i=0; i<8; i++) {//drawPiecesOnSquares
            for(int y=0; y<8; y++) {
                if(this.squares[i][y].piece != null) {
                   this.squares[i][y].piece.draw(this.getGraphics());//draw image of Piece
                }
            }
        }//--endOf--drawPiecesOnSquares
        if((this.active_x_square != 0) && (this.active_y_square != 0)) {//if some square is active
            this.getGraphics().drawImage(sel_square,((this.active_x_square-1)*(int)square_height),
                            ((this.active_y_square-1)*(int)square_height),null);//draw image of selected square
            this.moves = this.squares[(int)(this.active_x_square-1)]
                    [(int)(this.active_y_square-1)].piece.allMoves();
            //movies = this.squares[(int)(this.active_x_square-1)]
              //      [(int)(this.active_y_square-1)].piece.allMovies();

            for(Iterator it = moves.iterator(); it.hasNext();) {
                Square sq = (Square)it.next();
                this.getGraphics().drawImage(able_square, sq.pozX*(int)square_height, sq.pozY*(int)square_height, null);
            }
         }
    }/*--endOf-draw--*/
    
    /**
     Annotations to superclass Game updateing and painting the crossboard
     */
    
    @Override
    public void update(Graphics g) {
        paint(g);
    }
    
    @Override
    public void paint(Graphics g)
    {
        g.drawImage(image,0,0,null);//draw an Image of chessboard
        for(int i=0; i<8; i++) {//drawPiecesOnSquares
            for(int y=0; y<8; y++) {
                if(this.squares[i][y].piece != null) {
                   this.squares[i][y].piece.draw(g);//draw image of Piece
                }
            }
        }//--endOf--drawPiecesOnSquares
        if((this.active_x_square != 0) && (this.active_y_square != 0)) {//if some square is active
            g.drawImage(sel_square,((this.active_x_square-1)*(int)square_height),
                            ((this.active_y_square-1)*(int)square_height),null);//draw image of selected square
            this.moves = this.squares[(int)(this.active_x_square-1)]
                    [(int)(this.active_y_square-1)].piece.allMoves();

            for(Iterator it = moves.iterator(); it.hasNext();) {
                Square sq = (Square)it.next();
                g.drawImage(able_square, sq.pozX*(int)square_height, sq.pozY*(int)square_height, null);
            }
         }
        //this.draw();
    }/*--endOf-paint--*/
}

