/*
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package jchess;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JScrollPane;
import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.FileReader;
import java.io.IOException;
import java.util.Timer;
import java.util.Date;
import java.util.Calendar;
import java.util.ArrayList;
import java.awt.GridBagLayout;

/** Class responsible for the starts of new games, loading games and saving it, and for ending it
 *  This class is also responsible for appoing player with have a move at the moment
 */
public class Game extends JPanel implements MouseListener
{
    public  Settings settings;
    public  boolean blockedChessboard;
    public  Chessboard chessboard;
    private Player activePlayer;
    public  GameClock gameClock;
    public  Client client;
    public  Moves moves;
    public Chat chat;
    private GridBagLayout gbl;
    private GridBagConstraints gbc;

    Game()
    {
        this.setLayout(null);
        this.moves  = new Moves(this);
        settings    = new Settings();
        chessboard  = new Chessboard(this.settings,this.moves);
        chessboard.setVisible(true);
        chessboard.setSize(Chessboard.img_height, Chessboard.img_widht);
        chessboard.addMouseListener(this);
        chessboard.setLocation(new Point(0,0));
        this.add(chessboard);

        gameClock = new GameClock(this);
        gameClock.setSize(new Dimension(200,100));
        gameClock.setLocation(new Point(500,0));
        this.add(gameClock);

        JScrollPane movesHistory = this.moves.getScrollPane();
        movesHistory.setSize(new Dimension(180,350));
        movesHistory.setLocation(new Point(500,121));
        this.add(movesHistory);

        this.chat = new Chat();
        this.chat.setSize(new Dimension(400,100));
        this.chat.setLocation(new Point(0,500));
        this.chat.setMinimumSize(new Dimension(400,100));

        this.blockedChessboard = false;
        this.setLayout(null);
    }

     /** Method to save actual state of game
     *   @param path path where to save accual state of game
     */
    public void saveGame(File path) {
        File file = path;
        FileWriter fileW = null;
        try {
            fileW = new FileWriter(file);
        } catch(java.io.IOException exc) {
            System.err.println("error creating fileWriter: "+exc);
            return;
        }
        Calendar cal = Calendar.getInstance();
        String str = new String("");
        String info= new String("[Event \"Game\"]\n[Date \""+cal.get(cal.YEAR)+"."+(cal.get(cal.MONTH)+1)+"."+cal.get(cal.DAY_OF_MONTH)+"\"]\n" +
                "[White \""+this.settings.playerWhite.name+"\"]\n[Black \""+this.settings.playerBlack.name+"\"]\n\n");
        str += info;
        str += this.moves.getMovesInString();
        try {
            fileW.write(str);
            fileW.flush();
            fileW.close();
        } catch(java.io.IOException exc) {
            System.out.println("error writing to file: "+exc);
            JOptionPane.showMessageDialog(this,"error writing to file: "+exc);
            return;
        }
        JOptionPane.showMessageDialog(this,"Game seems to be saved properly.");
    }

    /** Loading game method(loading game state from the earlier saved file)
     *  @param file file to load (have to be opened outside this method)
     */
    static public void loadGame(File file) {
        FileReader fileR = null;
        try {
           fileR  = new FileReader(file);
        } catch(java.io.IOException exc) {
            System.out.println("Something wrong reading file: "+exc);
            return;
        }
        BufferedReader br = new BufferedReader(fileR);
        String tempStr    = new String();
        String blackName, whiteName;
        try {
            tempStr     = getLineWithVar(br, new String("[White"));
            whiteName   = getValue(tempStr);
            tempStr     = getLineWithVar(br, new String("[Black"));
            blackName   = getValue(tempStr);
            tempStr     = getLineWithVar(br, new String("1."));
        } catch(ReadGameError err) {
            System.out.println("Error reading file: "+err);
            return;
        }
        Game newGUI  = JChessApp.jcv.addNewTab(whiteName+" vs. "+blackName);
        Settings locSetts = newGUI.settings;
        locSetts.playerBlack.name = blackName;
        locSetts.playerWhite.name = whiteName;
        locSetts.playerBlack.setType(Player.playerTypes.localUser);
        locSetts.playerWhite.setType(Player.playerTypes.localUser);
        locSetts.gameMode = Settings.gameModes.loadGame;
        locSetts.gameType = Settings.gameTypes.local;
        
        newGUI.newGame();
        newGUI.blockedChessboard = true;
        newGUI.moves.setMoves(tempStr);
        newGUI.blockedChessboard = false;
        newGUI.chessboard.draw();
    }

    /** Method checking in with of line there is an error
     *  @param  br BufferedReader class object to operate on
     *  @param  srcStr String class object with text which variable you want to get in file
     *  @return String with searched variable in file (whole line)
     *  @throws ReadGameError class object when something goes wrong when reading file
     */
    static public String getLineWithVar(BufferedReader br, String srcStr) throws ReadGameError {
        String str = new String();
        while(true) {
            try {
                str = br.readLine();
            } catch(java.io.IOException exc) {
                System.out.println("Something wrong reading file: "+exc);
            }
            if(str == null) throw new ReadGameError();
            if(str.startsWith(srcStr)) {
                return str;
            }
        }
    }

    /** Method to get value from loaded txt line
     *  @return  string with loaded value
     *  @throws  ReadGameError object class when something goes wrong
     */
    static public String getValue(String line) throws ReadGameError {
        //System.out.println("getValue called with: "+line);
        int from    = line.indexOf("\"");
        int to      = line.lastIndexOf("\"");
        int size    = line.length()-1;
        String result = new String();
        if(to<from || from>size || to>size || to <0 || from< 0) {
            throw new ReadGameError();
        }
        try {
            result = line.substring(from+1, to);
        } catch(java.lang.StringIndexOutOfBoundsException exc) {
            System.out.println("error getting value: "+exc);
            return "none";
        }
        return result;
    }


    /** Method to Start new game
     */
    public void newGame() {
        chessboard.setPieces("", settings.playerWhite, settings.playerBlack);

        //System.out.println("new game, game type: "+settings.gameType.name());

        activePlayer = settings.playerWhite;
        if(activePlayer.playerType != Player.playerTypes.localUser)
            this.blockedChessboard = true;
    }

    /** Method to end game
     *  @param message what to show player(s) at end of the game (for example "draw", "black wins" etc.)
     */
    public void endGame(String massage) {
        this.blockedChessboard=true;
        System.out.println(massage);
        JOptionPane.showMessageDialog(null, massage);
    }

    /** Method to swich active players after move
     */
    public void switchActive()
    {
        if(activePlayer == settings.playerWhite)
            activePlayer = settings.playerBlack;      
        else
            activePlayer = settings.playerWhite;
        
        this.gameClock.switch_clocks();
    }

    /** Method of getting accualy active player
     *  @return  player, whose have a move
     */
    public Player getActivePlayer() {
        return this.activePlayer;
    }

    /** Method to go to next move (checks if game is local/network etc.)
     */
    public void nextMove()
    {
        switchActive();

        System.out.println("next move, active player: "+activePlayer.name+", color: "+activePlayer.color.name()+", type: "+activePlayer.playerType.name());
        if(activePlayer.playerType == Player.playerTypes.localUser)
        {
            this.blockedChessboard = false;
        }
        else if(activePlayer.playerType == Player.playerTypes.networkUser)
        {
            this.blockedChessboard = true;
        }
        else if(activePlayer.playerType == Player.playerTypes.computer)
        {}
    }

    /** Method to simulate Move to check if it's correct etc. (usable for network game).
     * @param beginX from which X (on chessboard) move starts
     * @param beginY from which Y (on chessboard) move starts
     * @param endX   to   which X (on chessboard) move go
     * @param endY   to   which Y (on chessboard) move go
     * */
    public boolean simulateMove(int beginX, int beginY, int endX, int endY) {
        chessboard.select(chessboard.squares[beginX][beginY]);
        if(chessboard.activeSquare.piece.allMoves().indexOf(chessboard.squares[endX][endY]) != -1) //move
        {
            chessboard.move(chessboard.squares[beginX][beginY], chessboard.squares[endX][endY]);
        }
        else
        {
            System.out.println("Bad move");
            return false;
        }
        chessboard.unselect();
        nextMove();
        
        return true;
    }

    // MouseListener:
    public void mouseClicked(MouseEvent arg0)
    {}

    public void mousePressed(MouseEvent event)
    { 
      if(event.getButton() == MouseEvent.BUTTON3) //right button
      {
          if(chessboard.undo())
              this.switchActive();
      }
      else //left button
      {
        if(!blockedChessboard)
        {
            int x = event.getX();//get X position of mouse
            int y = event.getY();//get Y position of mouse

            Square sq = chessboard.getSquare(x, y);
            if( (sq.piece == null && chessboard.activeSquare == null) ||
                    ( this.chessboard.activeSquare == null && sq.piece != null && sq.piece.player != this.activePlayer) ) return;
            
            if(sq.piece != null && sq.piece.player == this.activePlayer && sq != chessboard.activeSquare) {
                chessboard.unselect();
                chessboard.select(sq);
            }
            else if(chessboard.activeSquare == sq) //unselect
            {
                chessboard.unselect();
            }
            else if(chessboard.activeSquare.piece.allMoves().indexOf(sq) != -1) //move
            {
                if(settings.gameType == Settings.gameTypes.local)
                {
                    chessboard.move(chessboard.activeSquare, sq);
                }
                else if(settings.gameType == Settings.gameTypes.network)
                {
                    client.sendMove(chessboard.activeSquare.pozX, chessboard.activeSquare.pozY, sq.pozX,sq.pozY);
                    chessboard.move(chessboard.activeSquare, sq);
                }

                chessboard.unselect();

                //switch player
                this.nextMove();

                //checkmate or stalemate
                King king;
                if(this.activePlayer == settings.playerWhite) king = chessboard.kingWhite;
                else king = chessboard.kingBlack;

                switch(king.isCheckmatedOrStalemated())
                {
                    case 1:
                        this.endGame("Checkmate! "+king.player.color.toString()+" player lose!");
                        break;
                    case 2:
                        this.endGame("Stalemate! Draw!");
                        break;
                }
            }
        } else if(blockedChessboard) {
            System.out.println("Chessboard is blocked");
        }
      }
    }

    public void mouseReleased(MouseEvent arg0)
    {}

    public void mouseEntered(MouseEvent arg0)
    {}

    public void mouseExited(MouseEvent arg0)
    {}
}

class ReadGameError extends Exception{}