/*
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>..
 */

package jchess;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;




/*
 * @author Mateusz Lach 
 * based on: http://forums.sun.com/thread.jspa?threadID=337070
 */

public class JChessTabbedPane extends JTabbedPane implements MouseListener {

    private TabbedPaneIcon  closeIcon       = null;
    private Boolean         addTabExist     = null;
    private Image           addIcon         = null;
    private Image           clickedAddIcon  = null;
    private Image           unclickedAddIcon= null;

    JChessTabbedPane() {
        super();
        addMouseListener(this);
        this.closeIcon   = new TabbedPaneIcon(closeIcon);
        this.addTabExist = false;
        this.unclickedAddIcon = GUI.loadImage("add-tab-icon.png");
        this.clickedAddIcon   = GUI.loadImage("clicked-add-tab-icon.png");

        this.setAutoscrolls(true);
        this.setTabLayoutPolicy(HORIZONTAL);
      
    }
    
    @Override
    public void addTab(String title, Component component) {
        this.addTab(title, component, null);
    }

    public void addTab(String title, Component component, Icon closeIcon) {
        if(this.addTabExist) {
            this.removePlusTab();
        }
        super.addTab(title, new TabbedPaneIcon(closeIcon), component);
        System.out.println("Present number of tabs: "+this.getTabCount());
        this.addPlusTab();
        super.addMouseListener(this);
        this.addIcon = this.unclickedAddIcon;
    }

    private void addPlusTab() {
        this.addTabExist = true;
        super.addTab("   ", new NullPanel());//spaces to fit tab to + icon
        this.setEnabledAt(this.getTabCount()-1, false);
        //this.setSelectedIndex(this.getSelectedIndex()-1);//set selected to newGame
    }

    private void removePlusTab() {
        this.addTabExist = false;
        int num = this.getTabCount();
        this.removeTabAt(num-1);
    }
    

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        if(this.addTabExist) {
            
        }
        if(this.addIcon != null && this.getTabCount() > 0) {
           Rectangle rect = this.getBoundsAt(this.getTabCount()-1);
           g.drawImage(this.addIcon, rect.x+4, rect.y+8, null);
        }
    }

    private void showNewGameWindow() {
        if(JChessApp.jcv.newGameFrame == null) {
            JChessApp.jcv.newGameFrame = new NewGameWindow();
        }
        JChessApp.getApplication().show(JChessApp.jcv.newGameFrame);
    }

    public void mouseClicked(MouseEvent e) {
        Rectangle rect;
        int tabNumber=getUI().tabForCoordinate(this, e.getX(), e.getY());
        if (tabNumber >= 0 && tabNumber < this.getTabCount()-1) {
            rect=((TabbedPaneIcon)getIconAt(tabNumber)).getBounds();
            if (rect.contains(e.getX(), e.getY())) {
                System.out.println("Removing tab with "+tabNumber+" number!...");
                this.removeTabAt(tabNumber);//remove tab
                if(this.getTabCount() == 1 && this.getTitleAt(this.getTabCount()-1).trim().equals("")) {
                    this.removePlusTab();//if only + tab is left, remove it
                }
            }
        } else {
            if(tabNumber == this.getTabCount()-1) {//if clicked in + tab
                this.addIcon = this.clickedAddIcon;
                System.out.println("newGame by + button");
                this.showNewGameWindow();
            }
        }
        System.out.println("x:" +e.getX()+" y: "+e.getY());
    }


    public void mouseEntered(MouseEvent e) {
        if(this.getUI().tabForCoordinate(this, e.getX(), e.getY()) == this.getTabCount()-1) {
                this.addIcon = this.clickedAddIcon;
            } else {
                this.addIcon = this.unclickedAddIcon;
        }
    }
    public void mouseExited(MouseEvent e) {}
    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {
        this.addIcon = this.unclickedAddIcon;
    }



}
class NullPanel extends JPanel {
    //NullPanel (nothing in) 4 + tab needs - we've to return JPanel, don't delete.
    NullPanel(boolean showNewGame) {
        if(JChessApp.jcv.newGameFrame != null && showNewGame)
            JChessApp.getApplication().show(JChessApp.jcv.newGameFrame);//run new Game Frame settings
    }//--endOf-NullPanel

    NullPanel() {
        if(JChessApp.jcv.newGameFrame != null)
            JChessApp.getApplication().show(JChessApp.jcv.newGameFrame);//run new Game Frame settings
    }
}


class TabbedPaneIcon implements Icon {
    private int x_pos;
    private int y_pos;
    private int width;
    private int height;
    private Icon fileIcon;

    public TabbedPaneIcon(Icon fileIcon) {
        this.fileIcon=fileIcon;
        width=16;
        height=16;
    }//--endOf-TabbedPaneIcon--

    public void paintIcon(Component c, Graphics g, int x, int y) {
        this.x_pos=x;
        this.y_pos=y;

        Color col=g.getColor();

        g.setColor(Color.black);
        int y_p=y+2;
        //draw X
        g.drawLine(x+3, y_p+3, x+10, y_p+10);
        g.drawLine(x+3, y_p+4, x+9, y_p+10);
        g.drawLine(x+4, y_p+3, x+10, y_p+9);
        g.drawLine(x+10, y_p+3, x+3, y_p+10);
        g.drawLine(x+10, y_p+4, x+4, y_p+10);
        g.drawLine(x+9, y_p+3, x+3, y_p+9);//end of draw X
        g.setColor(col);
        if (fileIcon != null) {
            fileIcon.paintIcon(c, g, x+width, y_p);
        }
    }//--endOf-PaintIcon--

    public int getIconWidth() {
        return width + (fileIcon != null? fileIcon.getIconWidth() : 0);
    }//--endOf-getIconWidth--

    public int getIconHeight() {
        return height;
    }//--endOf-getIconHeight()--

    public Rectangle getBounds() {
        return new Rectangle(x_pos, y_pos, width, height);
    }
}