/**
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package jchess;
import java.util.ArrayList;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.*;
import java.awt.Dimension;
import java.awt.Rectangle;
import javax.swing.JOptionPane;
/** Class representing the history of players moves
 */
public class Moves extends AbstractTableModel {
    private ArrayList<String> move = new ArrayList<String>();
    private int columns_num = 3;
    private int rows_num = 0;
    private String[]   names = new String[] {"białe", "czarne"};
    private MyDefaultTableModel tableModel;
    private JScrollPane scrollPane;
    private JTable  table;
    private boolean enterBlack = false;
    private Game    game;

    Moves(Game game) {
        super();
        this.tableModel = new MyDefaultTableModel();
        this.table      = new JTable(this.tableModel);
        this.scrollPane = new JScrollPane(this.table);
        this.scrollPane.setMaximumSize(new Dimension(100,100));
        this.table.setMinimumSize(new Dimension(100,100));
        this.game       = game;

        this.tableModel.addColumn(this.names[0]);
        this.tableModel.addColumn(this.names[1]);
        this.addTableModelListener(null);
        this.tableModel.addTableModelListener(null);
        this.scrollPane.setAutoscrolls(true);
    }
    
    
    public void draw() {}

    @Override
    public String getValueAt(int x, int y) {
        return this.move.get((y*2)-1+(x-1));
    }

    @Override
    public int getRowCount() { return this.rows_num; }
    @Override
    public int getColumnCount() { return this.columns_num; }

    private void addRow() {
        this.tableModel.addRow(new String[2]);
    }

    public void addCastling(String move) {
        this.move.remove(this.move.size()-1);//remove last element (move of Rook)
        if(!this.enterBlack) {
            this.tableModel.setValueAt(move, this.tableModel.getRowCount()-1, 1);//replace last value
        } else {
            this.tableModel.setValueAt(move, this.tableModel.getRowCount()-1, 0);//replace last value
        }
        this.move.add(move);//add new move (O-O or O-O-O)
    }

    @Override
    public boolean isCellEditable(int a, int b)  {
        return false;
    }

    /** Method of adding new moves to the table
     */
    private void addMove2Table(String str) {
        if(!this.enterBlack) {
            this.addRow();
            this.tableModel.setValueAt(str, rows_num, 0);
        } else {
            this.tableModel.setValueAt(str, rows_num, 1);
            this.rows_num += 1;
        }
        this.enterBlack = !this.enterBlack;
        this.table.scrollRectToVisible(table.getCellRect(table.getRowCount()-1, 0, true));//scroll to down
    }

    /** Method of adding new move
    */
    public void addMove(String move) {
        if(isMoveCorrect(move)) {
            this.move.add(move);
            this.addMove2Table(move);
        }
        
    }
    
    public void addMove(Square begin, Square end) {
        String locMove = new String(begin.piece.symbol);
        locMove += new String(Character.toString((char)(begin.pozX+97)));//add letter of Square from which move was made
        locMove += new String(Integer.toString(begin.pozY+1));//add number of Square from which move was made
        if(end.piece != null) {
            locMove += "x";//take down opponent piece
        } else {
            locMove += "-";//normal move
        }
        locMove += new String(Character.toString((char)(end.pozX+97)));//add letter of Square to which move was made
        locMove += new String(Integer.toString(end.pozY+1));//add number of Square to which move was made
        if(begin.piece.symbol.equals("") && begin.pozX-end.pozX!=0 && end.piece==null) {
            locMove += "(e.p)";//pawn take down opponent en passant
        }
        if( (!this.enterBlack && this.game.chessboard.kingBlack.isChecked()) ||
                (this.enterBlack && this.game.chessboard.kingWhite.isChecked()) ) {//if checked
            
            if( (!this.enterBlack && this.game.chessboard.kingBlack.isCheckmatedOrStalemated()==1) ||
                (this.enterBlack && this.game.chessboard.kingWhite.isCheckmatedOrStalemated()==1) ) {//check if checkmated
                locMove += "#";//check mate
            } else {
                locMove += "+";//check
            }
        }
        this.move.add(locMove);
        this.addMove2Table(locMove);
        this.scrollPane.scrollRectToVisible(new Rectangle(0,this.scrollPane.getHeight()-2,1,1));
    }

    public JScrollPane getScrollPane() { 
        return this.scrollPane;
    }

    public ArrayList<String> getMoves() {
        return this.move;
    }

    /** Method with is checking is the move is correct
     *  @return boolean 1 if the move is correct, else 0
     */
    static public boolean isMoveCorrect(String move) {
        if(move.equals("O-O") || move.equals("O-O-O")) return true;
        int from = 0;
        int sign = move.charAt(from);//get First
        switch(sign) { //if sign of piece, get next
            case 66: // B like Bishop
            case 75: // K like King
            case 78: // N like Knight
            case 81: // Q like Queen
            case 82: from = 1; break; // R like Rook
        }
        sign = move.charAt(from);
        System.out.println(sign);
        if(sign < 97 || sign > 104)   return false;
        sign = move.charAt(from+1);
        if(sign < 49 || sign > 56)    return false;
        sign = move.charAt(from+2);
        if(sign != 45 && sign != 120) return false;
        sign = move.charAt(from+3);
        if(sign < 97 || sign > 104)   return false;
        sign = move.charAt(from+4);
        if(sign < 49 || sign > 56)    return false;
        return true;
    }

    public void addMoves(ArrayList<String> list) {
        for(String singleMove : list) {
            if(isMoveCorrect(singleMove)) this.addMove(singleMove);
        }
    }

    /** Method of getting the moves in string
     *  @return string with saved moves
     */
    public String getMovesInString() {
        int n = 1;
        int i = 0;
        String str = new String();
        for(String locMove : this.getMoves()) {
            if(i % 2 == 0) {
                str += n+". ";
                n += 1;
            }
            str += locMove+" ";
            i += 1;
        }
        return str;
    }

    /** Method to set all moves from String with validation test (usefoul for network game)
     *  @param  String moves to set in String like PGN with full-notation format
     */
    public void setMoves(String moves) {
        int from    = 0;
        int to      = 0;
        int n       = 1;
        ArrayList<String> tempArray = new ArrayList();
        int tempStrSize = moves.length()-1;
        while(true) {
            from = moves.indexOf(" ", from);
            to   = moves.indexOf(" ", from+1);
            //System.out.println(from+">"+to);
            try {
                tempArray.add(moves.substring(from+1, to).trim());
            } catch(java.lang.StringIndexOutOfBoundsException exc) {
                System.out.println("error parsing file to load: "+exc);
                break;
            }
            if(n % 2 == 0) {
                from = moves.indexOf(".", to);
                if(from < to) break;
            } else {
                from = to;
            }
            n += 1;
            if(from > tempStrSize || to > tempStrSize) break;
        }
        for(String locMove: tempArray) {//test if moves are written correctly
            if(!Moves.isMoveCorrect(locMove.trim())) {//if not
                JOptionPane.showMessageDialog(this.game, "Loaded file is not valid JChess file! Stoppen on move: "+move);
                return;//show message and finish reading game
            }
        }
        boolean canMove = false;
        for(String locMove: tempArray) {
            if(locMove.equals("O-O-O") || locMove.equals("O-O")) { //if castling
                int[] values = new int[4];
                if(locMove.equals("O-O-O")) {
                    if(this.game.getActivePlayer().color == Player.colors.black) { //if black turn
                        values = new int[]{4,0,2,0};//move value for castling (King move)
                    } else {
                        values = new int[]{4,7,2,7};//move value for castling (King move)
                    }
                } else if(locMove.equals("O-O")) { //if short castling
                    if(this.game.getActivePlayer().color == Player.colors.black) {//if black turn
                        values = new int[]{4,0,6,0};//move value for castling (King move)
                    } else {
                        values = new int[]{4,7,6,7};//move value for castling (King move)
                    }
                }
                canMove = this.game.simulateMove(values[0], values[1], values[2], values[3]);
                if(!canMove) {//if move is illegal
                    JOptionPane.showMessageDialog(this.game, "IllegalMove on: "+locMove);
                    return;//finish reading game and show message
                }
                continue;
            }
            from = 0;
            int num = locMove.charAt(from);
            if(num <= 90 && num >= 65) from = 1;
            int xFrom = locMove.charAt(from)-97;//from ASCII
            int yFrom = locMove.charAt(from+1)-49;//from ASCII
            int xTo   = locMove.charAt(from+3)-97;//from ASCII
            int yTo   = locMove.charAt(from+4)-49;//from ASCII
            canMove = this.game.simulateMove(xFrom, yFrom, xTo, yTo);
            if(!canMove) {//if move is illegal
                JOptionPane.showMessageDialog(this.game, "IllegalMove on: "+locMove);
                return;//finish reading game and show message
            }
        }
    }
}
     /*
     * Overriding DefaultTableModel and  isCellEditable method
     * (history cannot be edited by player)
     */
class MyDefaultTableModel extends DefaultTableModel {
   
    MyDefaultTableModel() {
        super();
    }

    @Override
    public boolean isCellEditable(int a, int b)  {
        return false;
    }
}