package jchess;

import java.awt.Graphics;
import java.awt.Image;

/*
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
import java.util.ArrayList;
import java.util.Iterator;
/**
 *
 * @author MarioX
 */
public abstract class Piece
{
    Chessboard chessboard; // <-- this relations isn't in class diagram, but it's necessary :/

    public Square square;
    public Player player;
    String name;
    protected String symbol;
    String imageBlack;
    String imageWhite;
    Image image = null;

    public static short value=0;
    /**
     Class to represent a piece (any kind) - this class should be extended to represent pawn, bishop etc.
     */
    Piece(Chessboard chessboard, Player player) {
        this.chessboard = chessboard;
        this.player     = player;
        if(player.color == player.color.black) {
            this.image = GUI.loadImage(imageBlack);
        } else {
            this.image = GUI.loadImage(imageWhite);
        }

    }
   /* Method to draw piece on chessboard
    * @graph : where to draw
    */
    void draw(Graphics g){
        int x = this.square.pozX*this.chessboard.get_square_height();
        int y = this.square.pozY*this.chessboard.get_square_height();
        if(image != null && g != null) {
            g.drawImage(image,x+10,y+3,null);
        } 
    }

    void clean(){
    }

    boolean canMove(Square square, ArrayList allmoves) {
        /* method check if Piece can move to given square
         * @square: where piece want to move (Square object)
         * @allmoves:
         * */
        //throw new UnsupportedOperationException("Not supported yet.");
        ArrayList moves = allmoves;
        for(Iterator it = moves.iterator(); it.hasNext();) {
                Square sq = (Square)it.next();//get next from iterator
                if(sq == square) {//if adress is the same
                    return true; //piece canMove
                }
        }
        return false;//if not, piece cannot move
    }

    void setImages(String white, String black) {
        /* method set image to black or white (depends on player color)
         * @white: String with name of image with white piece
         * @black: String with name of image with black piece
         * */
        this.imageBlack = black;
        this.imageWhite = white;
        if(player.color == player.color.black) {
            this.image = GUI.loadImage(imageBlack);
        } else {
            this.image = GUI.loadImage(imageWhite);
        }
    }/*--endOf-setImages(String white, String black)--*/

    abstract public ArrayList allMoves();

        //extras methods:
    protected boolean isout(int x, int y) {
        /* Method is useful for out of bounds protection
         * @x: x position on chessboard
         * @y: y position on chessboard
         * @return: true if parameters are out of bounds (array)
         * */
        if(x<0 || x>7 || y<0 || y>7)
            return true;

        return false;
    }

    protected boolean checkPiece(int x, int y) {
        /* 
         * @x: x position on chessboard
         * @y: y position on chessboard
         * @return: true if can move
         * */
        if(chessboard.squares[x][y].piece != null &&
                chessboard.squares[x][y].piece.name.equals("King")) {
                return false;
        }
        Piece piece = chessboard.squares[x][y].piece;
        if(piece == null || //if this sqhuare is empty
                piece.player != this.player) //or piece is another player
        {
                return true;
        } 
        return false;
    }

    protected boolean otherOwner(int x, int y) {
        /* Method check if piece has other owner than calling piece
         * @x: x position on chessboard
         * @y: y position on chessboard
         * @return: true if owner(player) is different
         * */
        Square sq = chessboard.squares[x][y];
        if(sq.piece == null) {
            return false;
        }
        if(this.player != sq.piece.player) {
            return true;
        }
        return false;
    }

    public String getSymbol() {
        return this.symbol;
    }
}
